! This file is main file of SusHi.
! 
! written by Robert Harlander, Stefan Liebler, Hendrik Mantler
! and Shruti Patel (SusHiMi)
! Bergische Universitaet Wuppertal, RWTH Aachen, KIT, DESY
! Version 1.7.0
! 
      program SUSHI

      implicit none
      integer ninp,noutp,i,j
      integer loopCPup,loopCP
      double precision alphasPDFm
      !Arrays for results from SusHi core runs (e.g. in case of CP violation)
      double precision ggchanA(0:2), qgchanA(0:2), qqchanA(0:2)
      double precision ggchanerrA(0:2), qgchanerrA(0:2), qqchanerrA(0:2)
      double precision xsecA(0:2,2), xsecerrA(0:2,2)
      double precision xeffA(0:2), xefferrA(0:2)
      double precision elwA(0:2)
      double precision SUgghA(0:2,4,4,-1:5),SUggherrA(0:2,4,4,-1:5)
      double precision SUbbhA(0:2,4), SUbbherrA(0:2,4)
      double precision xeffvarA(0:2,1:2),gghXSvarA(0:2,1:2)
      !Temporary arrays to avoid runtime warnings in coresushi calls:
      double precision SUgghtmp(1:4,1:4,-1:5), SUggherrtmp(1:4,1:4,-1:5)
      double precision SUbbhtmp(1:4), SUbbherrtmp(1:4)
      double precision xsectmp(1:2), xsecerrtmp(1:2)
      double precision Xeffvartmp(1:2), gghXSvartmp(1:2)
      !Arrays for couplings
      double complex gb13A(2), gbh123A(2), gbh213A(2), gb23A(2)
      double complex gt13A(2), gth123A(2), gth213A(2), gt23A(2)
      double complex gbA(2), gcA(2), gtA(2)
      !Interference
      double precision etaA(2,8),BRA(8)

      !common variables used by various routines
      include 'commons/common-inputoutput.f'
      include 'commons/common-consts.f'
      include 'commons/common-vars.f'
      include 'commons/common-keys.f'
      include 'commons/common-quark.f'
      include 'commons/common-int.f'
      include 'commons/common-ren.f'
      include 'commons/common-citations.f'
      include 'commons/common-flags.f'

      common /vgasio/ ninp,noutp

      call get_command_argument(1,jfilein)
      call get_command_argument(2,jfnameact)

      if ((len_trim(jfilein).eq.0).or.(len_trim(jfnameact).eq.0)) then
       write(*,*) "SusHi must be called with two arguments:"
       write(*,*) "./SusHi in.in out.out"
       write(*,*) "The input-file in.in has to be provided"
       write(*,*) "in the SLHA-standard. Example files: SM.in, ..."
       stop
      end if

      extflag = .false.

      call initsushi(htlflag)
      
      if (verbose.ne.0) then
         write(*,105)
         write(*,106) 'Higgs mass:',Mh
         write(*,105)
      endif

      xsecA = 0.d0
      xsecerrA = 0.d0
      xeffA = 0.d0
      xefferrA = 0.d0
      SUbbhA = 0.d0
      SUbbherrA = 0.d0
      ggchanA = 0.d0
      qgchanA = 0.d0
      qqchanA = 0.d0
      ggchanerrA = 0.d0
      qgchanerrA = 0.d0
      qqchanerrA = 0.d0
      SUgghA = 0.d0
      SUggherrA = 0.d0
      elwA = 0.d0

      !VEGAS output
      noutp = 6
      
      call cutssushi() !defined in src/sushi/sushicore.f

C-{{{ Calculation of cross sections
!     parameters which differ compared to single runs:
!      mh, mh2, z, muR, muF, lfh, lfr, lrt, muB
!      minpt, maxpt, minrap, maxrap, pt, y, minptc, maxptc

      loopCPup = 1
      if (CPvio.eq.1) then
        write(*,105)
        write(*,*) "Running SusHi twice due to CP violation!"
        write(*,105)
        loopCPup = 2
      end if
      do loopCP = 1,loopCPup  !CP violation loop
       if ((CPvio.eq.1).and.(loopCP.eq.1)) pseudo = 0
       if ((CPvio.eq.1).and.(loopCP.eq.2)) pseudo = 1
       !defined in src/sushi/sushicore.f:
       call coresushi(pseudo,SUgghtmp,SUggherrtmp,SUbbhtmp,SUbbherrtmp
     &      ,ggchanA(loopCP),qgchanA(loopCP),qqchanA(loopCP),
     &      ggchanerrA(loopCP),qgchanerrA(loopCP),qqchanerrA(loopCP),
     &      xsectmp,xeffA(loopCP), xsecerrtmp,xefferrA(loopCP),
     &      Xeffvartmp,gghXSvartmp,elwA(loopCP), gb13A(loopCP)
     &      ,gbh123A(loopCP),gbh213A(loopCP),gb23A(loopCP),
     &      gt13A(loopCP),gth123A(loopCP),gth213A(loopCP),gt23A(loopCP),
     &      gbA(loopCP),gtA(loopCP))
       gcA(loopCP) = gtA(loopCP)
       SUgghA(loopCP,1:4,1:4,-1:5) = SUgghtmp(1:4,1:4,-1:5)
       SUggherrA(loopCP,1:4,1:4,-1:5) = SUggherrtmp(1:4,1:4,-1:5)
       SUbbhA(loopCP,1:4) = SUbbhtmp(1:4)
       SUbbherrA(loopCP,1:4) = SUbbherrtmp(1:4)
       xsecA(loopCP,1:2) = xsectmp(1:2)
       xsecerrA(loopCP,1:2) = xsecerrtmp(1:2)
       XeffvarA(loopCP,1:2) = Xeffvartmp(1:2)
       gghXSvarA(loopCP,1:2) = gghXSvartmp(1:2)
      end do

      !sum/process the obtained results and save them on 0-th position of arrays:
      elwA(0) = elwA(1)
      xeffvarA(0,1:2) = xeffvarA(1,1:2)
      do loopCP=1,loopCPup
      ggchanA(0) = ggchanA(0) + ggchanA(loopCP)
      qgchanA(0) = qgchanA(0) + qgchanA(loopCP)
      qqchanA(0) = qqchanA(0) + qqchanA(loopCP)
      ggchanerrA(0) = ggchanerrA(0) + ggchanerrA(loopCP)
      qgchanerrA(0) = qgchanerrA(0) + qgchanerrA(loopCP)
      qqchanerrA(0) = qqchanerrA(0) + qqchanerrA(loopCP)
      xsecA(0,1:2) = xsecA(0,1:2) + xsecA(loopCP,1:2)
      xsecerrA(0,1:2) = xsecerrA(0,1:2) + xsecerrA(loopCP,1:2)
      xeffA(0) = xeffA(0) + xeffA(loopCP)
      xefferrA(0) = xefferrA(0) + xefferrA(loopCP)
      SUbbhA(0,1:3) = SUbbhA(0,1:3) + SubbhA(loopCP,1:3)
      SUbbherrA(0,1:3) = SUbbherrA(0,1:3) + SubbherrA(loopCP,1:3)
      !Add up the absolute errors in quadrature for analytic muR variation:
      gghXSvarA(0,1:2) = gghXSvarA(0,1:2) + gghXSvarA(loopCP,1:2)**2
      end do
      gghXSvarA(0,1) = -dsqrt(gghXSvarA(0,1))
      gghXSvarA(0,2) = dsqrt(gghXSvarA(0,2))
C-}}}
C-{{{
#ifdef FHINT
      etaA=0.d0
      BRA=0.d0
      citations(60) = 1
      !if (Sind.ge.2) then !no implementation for h yet
      call runFHBRandINT(alfasmz,mt,mbmb,mw,mz
     &,cAt,cAb,CmuSUSY,M_3,etaA,BRA) !providing XS and index of Higgs
      !end if
#endif
C-}}}
C-{{{ showing screen output and writing the output-file
      call screenoutputsushi(ggchanA,qgchanA,qqchanA,
     & ggchanerrA,qgchanerrA,qqchanerrA,
     & SUbbhA,SUbbherrA,
     & SUgghA,SUggherrA,
     & xsecA,xsecerrA,xeffA,xefferrA,
     & xeffvarA,gghXSvarA)
      if (scalespt.and.(norderggh.ge.1)) then
         call SUinitpdf(2,SUpdfs(2),SUiset(2))
         awithpt = alphasPDFm(2,muRfacggh*dsqrt(mh**2+pt**2))
         awopt = alphasPDFm(2,murggh)
      endif
      call outputsushi(ggchanA,qgchanA,qqchanA,
     & ggchanerrA,qgchanerrA,qqchanerrA,
     & SUbbhA,SUbbherrA,
     & SUgghA,SUggherrA,
     & xsecA,xsecerrA,xeffA,xefferrA,
     & xeffvarA,gghXSvarA,
     & gb13A,gbh123A,gbh213A,gb23A,gt13A,
     & gth123A,gth213A,gt23A,gbA,gcA,gtA,elwA,
     & etaA,BRA)

      write(*,104) " Results written in output file: ",jfnameact
      write(*,*) achar(27)//'[32mMake sure to cite the references listed
     &'//' in output file '//trim(jfnameact)//'!'//achar(27)//'[0m'
      stop
      call printcopysushi(0,6,verbose)

 104  format(a,a)
 105  format('#--------------------------------------------------#')
 106  format(a12,'    ',f10.5, '   GeV')
C-}}}

      end

!end of file
